#include <c8051f330.h>
#include "lcd_3310_lib.h"

unsigned char xdata buffer[504];
unsigned char cur_x = 0, cur_y = 0;

void lcd_send_byte(unsigned char byte)
{
	unsigned char n;
	
	sce = 0;
	sclk = 0;

	for (n = 0; n < 8; ++n)
	{
		if (byte & 0x80)
			sdin = 1;
		else
			sdin = 0;
 
		byte <<= 1;

		sclk = 1; 
		sclk = 1;
		sclk = 1;
		sclk = 0; // strobe clock	
		sclk = 0;
		sclk = 0;
	}
	
	sce = 1;
}

void lcd_send_cmd(unsigned char byte)
{
	dc = 0;
	lcd_send_byte(byte);
}

void lcd_send_data(unsigned char byte)
{
	dc = 1;
	lcd_send_byte(byte);
	++cur_x;
}

void lcd_init()
{
	sdin = 0;
	sclk = 0;
	sce = 0;
	dc = 0;
	res = 0;
	lcd_delay();
	res = 1;
	sce = 1;

	lcd_send_cmd(0x21);
	lcd_send_cmd(0xc8);
	lcd_send_cmd(0x06);
	lcd_send_cmd(0x13);
	lcd_send_cmd(0x20);
	lcd_send_cmd(0x21);
	lcd_send_cmd(0x0c);

}

void lcd_delay()
{
	unsigned long bla;
	for(bla = 0; bla < 100000; ++bla);
}

void lcd_set_cursor(unsigned char x, unsigned char y)
{
	cur_x = x;
	cur_y = y;
}

void lcd_send_char(unsigned char byte, unsigned char font_size)
{
	unsigned char x, y;
	unsigned int temp;

	for(x = 0; x < font_sizes[font_size][0]; ++x)
	{	
		if (font_size == 0)
		{
			temp = (unsigned char)font_5x8[byte-32][x];
		}else if (font_size == 1)
		{
			temp = font_5x12[byte-32][x];
		}else if (font_size == 2)
		{
			temp = font_7x12[byte-32][x];
		}else if (font_size == 3)
		{
			temp = font_10x16[byte-32][x];
		}else if (font_size == 4)
		{
			temp = font_12x16[byte-32][x];
		}

		for(y = 0; y < font_sizes[font_size][1]; ++y)
		{
			lcd_set_pixel((unsigned char)(0x0001 & temp), cur_x, (cur_y+y));
			temp >>= 1;
		}
		cur_x++;		
	}

	for(y = 0; y < font_sizes[font_size][1]; ++y)
		lcd_set_pixel(0, cur_x, cur_y+y);
	++cur_x;
}

void lcd_send_string(unsigned char *string, unsigned char font_size)
{
	while(*string)
		lcd_send_char(*string++, font_size);

}

void lcd_clear_screen()
{
	unsigned int n;

	for(n = 0; n < 504; ++n)
		buffer[n] = 0;
}

void lcd_flush()
{
	unsigned int n;
	
	lcd_send_cmd(0x80);
	lcd_send_cmd(0x40);

	for(n = 0; n < 504; ++n)
		lcd_send_data(buffer[n]);
}

void lcd_set_pixel(unsigned char set_or_not, unsigned char x, unsigned char y)
{
	unsigned char n_shift;
	unsigned int n;

	n_shift = y & 0x07;
	y = y >> 3;

	n = y * 84 + x;
	
	// set or clear bit
	buffer[n] = (buffer[n] | (set_or_not << n_shift)) & (0xff ^ ((set_or_not ^ 0x01) << n_shift));
}


void lcd_draw_line(int x0, int y0, int x1, int y1) {
	int dy = y1 - y0;
	int dx = x1 - x0;
	int stepx, stepy;

	if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; }
	if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }
	dy <<= 1;
	dx <<= 1;

	lcd_set_pixel(1, x0, y0);
	if (dx > dy) {
		int fraction = dy - (dx >> 1);
		while (x0 != x1) {
			if (fraction >= 0) {
				y0 += stepy;
				fraction -= dx;
			}
			x0 += stepx;
			fraction += dy;
			lcd_set_pixel(1, x0, y0);
		}
	} else {
		int fraction = dx - (dy >> 1);
		while (y0 != y1) {
			if (fraction >= 0) {
				x0 += stepx;
				fraction -= dy;
			}
			y0 += stepy;
			fraction += dx;
			lcd_set_pixel(1, x0, y0);
		}
	}
}



// width, height
unsigned char code font_sizes[][2] = 
{
	{5, 8},
	{5, 12},
	{7, 12},
	{10, 16},
	{12, 16}
};

unsigned char code font_5x8[][5] = {
    { 0x00, 0x00, 0x00, 0x00, 0x00 },   // sp
    { 0x00, 0x00, 0x2f, 0x00, 0x00 },   // !
    { 0x00, 0x07, 0x00, 0x07, 0x00 },   // "
    { 0x14, 0x7f, 0x14, 0x7f, 0x14 },   // #
    { 0x24, 0x2a, 0x7f, 0x2a, 0x12 },   // $
    { 0x32, 0x34, 0x08, 0x16, 0x26 },   // %
    { 0x36, 0x49, 0x55, 0x22, 0x50 },   // &
    { 0x00, 0x05, 0x03, 0x00, 0x00 },   // '
    { 0x00, 0x1c, 0x22, 0x41, 0x00 },   // (
    { 0x00, 0x41, 0x22, 0x1c, 0x00 },   // )
    { 0x14, 0x08, 0x3E, 0x08, 0x14 },   // *
    { 0x08, 0x08, 0x3E, 0x08, 0x08 },   // +
    { 0x00, 0x00, 0x50, 0x30, 0x00 },   // ,
    { 0x10, 0x10, 0x10, 0x10, 0x10 },   // -
    { 0x00, 0x60, 0x60, 0x00, 0x00 },   // .
    { 0x20, 0x10, 0x08, 0x04, 0x02 },   // /
    { 0x3E, 0x51, 0x49, 0x45, 0x3E },   // 0
    { 0x00, 0x42, 0x7F, 0x40, 0x00 },   // 1
    { 0x42, 0x61, 0x51, 0x49, 0x46 },   // 2
    { 0x21, 0x41, 0x45, 0x4B, 0x31 },   // 3
    { 0x18, 0x14, 0x12, 0x7F, 0x10 },   // 4
    { 0x27, 0x45, 0x45, 0x45, 0x39 },   // 5
    { 0x3C, 0x4A, 0x49, 0x49, 0x30 },   // 6
    { 0x01, 0x71, 0x09, 0x05, 0x03 },   // 7
    { 0x36, 0x49, 0x49, 0x49, 0x36 },   // 8
    { 0x06, 0x49, 0x49, 0x29, 0x1E },   // 9
    { 0x00, 0x36, 0x36, 0x00, 0x00 },   // :
    { 0x00, 0x56, 0x36, 0x00, 0x00 },   // ;
    { 0x08, 0x14, 0x22, 0x41, 0x00 },   // <
    { 0x14, 0x14, 0x14, 0x14, 0x14 },   // =
    { 0x00, 0x41, 0x22, 0x14, 0x08 },   // >
    { 0x02, 0x01, 0x51, 0x09, 0x06 },   // ?
    { 0x32, 0x49, 0x59, 0x51, 0x3E },   // @
    { 0x7E, 0x11, 0x11, 0x11, 0x7E },   // A
    { 0x7F, 0x49, 0x49, 0x49, 0x36 },   // B
    { 0x3E, 0x41, 0x41, 0x41, 0x22 },   // C
    { 0x7F, 0x41, 0x41, 0x22, 0x1C },   // D
    { 0x7F, 0x49, 0x49, 0x49, 0x41 },   // E
    { 0x7F, 0x09, 0x09, 0x09, 0x01 },   // F
    { 0x3E, 0x41, 0x49, 0x49, 0x7A },   // G
    { 0x7F, 0x08, 0x08, 0x08, 0x7F },   // H
    { 0x00, 0x41, 0x7F, 0x41, 0x00 },   // I
    { 0x20, 0x40, 0x41, 0x3F, 0x01 },   // J
    { 0x7F, 0x08, 0x14, 0x22, 0x41 },   // K
    { 0x7F, 0x40, 0x40, 0x40, 0x40 },   // L
    { 0x7F, 0x02, 0x0C, 0x02, 0x7F },   // M
    { 0x7F, 0x04, 0x08, 0x10, 0x7F },   // N
    { 0x3E, 0x41, 0x41, 0x41, 0x3E },   // O
    { 0x7F, 0x09, 0x09, 0x09, 0x06 },   // P
    { 0x3E, 0x41, 0x51, 0x21, 0x5E },   // Q
    { 0x7F, 0x09, 0x19, 0x29, 0x46 },   // R
    { 0x46, 0x49, 0x49, 0x49, 0x31 },   // S
    { 0x01, 0x01, 0x7F, 0x01, 0x01 },   // T
    { 0x3F, 0x40, 0x40, 0x40, 0x3F },   // U
    { 0x1F, 0x20, 0x40, 0x20, 0x1F },   // V
    { 0x3F, 0x40, 0x38, 0x40, 0x3F },   // W
    { 0x63, 0x14, 0x08, 0x14, 0x63 },   // X
    { 0x07, 0x08, 0x70, 0x08, 0x07 },   // Y
    { 0x61, 0x51, 0x49, 0x45, 0x43 },   // Z
    { 0x00, 0x7F, 0x41, 0x41, 0x00 },   // [
    { 0x55, 0x2A, 0x55, 0x2A, 0x55 },   // 55
    { 0x00, 0x41, 0x41, 0x7F, 0x00 },   // ]
    { 0x0f, 0x10, 0xff, 0x11, 0x0e },   // phi  // original: { 0x04, 0x02, 0x01, 0x02, 0x04 },   // ^
    { 0x40, 0x40, 0x40, 0x40, 0x40 },   // _
    { 0x00, 0x01, 0x02, 0x04, 0x00 },   // '
    { 0x20, 0x54, 0x54, 0x54, 0x78 },   // a
    { 0x7F, 0x48, 0x44, 0x44, 0x38 },   // b
    { 0x38, 0x44, 0x44, 0x44, 0x20 },   // c
    { 0x38, 0x44, 0x44, 0x48, 0x7F },   // d
    { 0x38, 0x54, 0x54, 0x54, 0x18 },   // e
    { 0x08, 0x7E, 0x09, 0x01, 0x02 },   // f
    { 0x0C, 0x52, 0x52, 0x52, 0x3E },   // g
    { 0x7F, 0x08, 0x04, 0x04, 0x78 },   // h
    { 0x00, 0x44, 0x7D, 0x40, 0x00 },   // i
    { 0x20, 0x40, 0x44, 0x3D, 0x00 },   // j
    { 0x7F, 0x10, 0x28, 0x44, 0x00 },   // k
    { 0x00, 0x41, 0x7F, 0x40, 0x00 },   // l
    { 0x7C, 0x04, 0x18, 0x04, 0x78 },   // m
    { 0x7C, 0x08, 0x04, 0x04, 0x78 },   // n
    { 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
    { 0x7C, 0x14, 0x14, 0x14, 0x08 },   // p
    { 0x08, 0x14, 0x14, 0x18, 0x7C },   // q
    { 0x7C, 0x08, 0x04, 0x04, 0x08 },   // r
    { 0x48, 0x54, 0x54, 0x54, 0x20 },   // s
    { 0x04, 0x3F, 0x44, 0x40, 0x20 },   // t
    { 0x3C, 0x40, 0x40, 0x20, 0x7C },   // u
    { 0x1C, 0x20, 0x40, 0x20, 0x1C },   // v
    { 0x3C, 0x40, 0x30, 0x40, 0x3C },   // w
    { 0x44, 0x28, 0x10, 0x28, 0x44 },   // x
    { 0x0C, 0x50, 0x50, 0x50, 0x3C },   // y
    { 0x44, 0x64, 0x54, 0x4C, 0x44 },   // z
    { 0x00, 0x7F, 0x3E, 0x1C, 0x08 },   // > Filled
    { 0x08, 0x1C, 0x3E, 0x7F, 0x00 },   // < Filled
    { 0x08, 0x7C, 0x7E, 0x7C, 0x08 },   // Arrow up
    { 0x10, 0x3E, 0x7E, 0x3E, 0x10 },   // Arrow down    
    { 0x3E, 0x3E, 0x3E, 0x3E, 0x3E },   // Stop
    { 0x00, 0x7F, 0x3E, 0x1C, 0x08 },   // Play
	{ 0x07, 0x40, 0xff, 0x88, 0x07 }   // phi
};

/*
unsigned int code font_5x12[][5] = {
{0,0,0,0,0},
{0,0,894,0,0},
{0,15,0,15,0},
{72,510,72,510,72},
{280,550,1604,392,0},
{774,230,792,774,0},
{476,546,604,384,576},
{0,0,11,7,0},
{0,508,514,0,0},
{0,514,508,0,0},
{72,48,252,48,72},
{32,32,248,32,32},
{0,2816,1792,0,0},
{32,32,32,32,0},
{0,768,768,0,0},
{768,224,24,6,0},
{1020,514,514,510,0},
{520,516,1022,512,0},
{908,578,546,540,0},
{260,546,546,476,0},
{96,88,68,1022,0},
{318,546,546,450,0},
{504,532,530,482,0},
{2,898,98,30,0},
{476,546,546,476,0},
{572,578,322,252,0},
{0,816,816,0,0},
{0,2864,1840,0,0},
{96,144,264,516,0},
{160,160,160,160,0},
{516,264,144,96,0},
{12,2,866,28,0},
{508,514,610,636,0},
{1020,34,34,1020,0},
{1022,546,546,476,0},
{508,514,514,268,0},
{1022,514,514,508,0},
{1022,546,546,546,0},
{1022,34,34,34,0},
{508,514,578,460,0},
{1022,32,32,1022,0},
{0,514,1022,514,0},
{384,514,510,2,0},
{1022,32,216,774,0},
{1022,512,512,512,0},
{1022,24,24,1022,0},
{1022,24,96,1022,0},
{508,514,514,508,0},
{1022,34,34,28,0},
{508,514,1538,2556,0},
{1022,98,162,796,0},
{412,546,546,460,0},
{2,2,1022,2,2},
{510,512,512,510,0},
{62,192,768,254,0},
{1022,128,128,1022,0},
{910,112,112,910,0},
{14,48,960,48,14},
{898,610,538,518,0},
{0,1022,514,0,0},
{6,24,224,768,0},
{0,514,1022,0,0},
{0,4,2,4,0},
{2048,2048,2048,2048,2048},
{0,7,11,0,0},
{384,592,592,992,0},
{1022,528,528,480,0},
{480,528,528,528,0},
{480,528,528,1022,0},
{480,656,656,736,0},
{16,1020,18,0,0},
{480,2576,2576,2032,0},
{1022,16,16,992,0},
{0,512,1012,512,0},
{1024,2048,2036,0,0},
{1022,192,288,528,0},
{0,514,1022,512,0},
{1008,32,32,1008,0},
{1008,32,16,992,0},
{480,528,528,480,0},
{4080,528,528,480,0},
{480,528,528,4080,0},
{16,992,16,48,0},
{288,592,656,288,0},
{16,1020,528,0,0},
{496,512,512,1008,0},
{48,192,768,240,0},
{1008,256,256,1008,0},
{816,192,192,816,0},
{2544,2560,2560,2032,0},
{784,656,592,560,0},
{32,476,514,0,0},
{0,0,1007,0,0},
{0,514,476,32,0},
{4,2,4,2,0},
{992,536,518,536,992}
};


unsigned int code font_7x12[][7] = {
{0,0,0,0,0,0,0},
{0,0,447,447,0,0,0},
{0,7,7,0,7,7,0},
{0,72,510,72,510,72,0},
{0,76,158,915,242,100,0},
{0,454,489,54,216,303,199},
{0,230,511,281,255,486,304},
{0,0,4,7,3,0,0},
{0,124,254,387,257,0,0},
{0,257,387,254,124,0,0},
{16,84,124,56,124,84,16},
{0,32,32,248,248,32,32},
{0,0,1408,1920,896,0,0},
{0,32,32,32,32,32,0},
{0,0,384,384,384,0,0},
{256,448,240,60,15,3,0},
{0,510,511,257,511,255,0},
{0,4,6,511,511,0,0},
{0,390,455,369,319,270,0},
{0,130,403,273,511,238,0},
{0,120,127,71,504,504,64},
{0,159,415,273,497,225,0},
{0,248,510,279,497,224,0},
{0,1,449,505,63,7,0},
{0,238,511,273,511,238,0},
{0,270,415,241,127,30,0},
{0,0,396,396,396,0,0},
{0,0,1420,1932,908,0,0},
{0,16,56,108,198,387,0},
{0,72,72,72,72,72,0},
{0,387,198,108,56,16,0},
{0,6,423,433,31,14,0},
{0,124,254,387,313,325,318},
{0,508,510,35,510,508,0},
{0,511,511,273,511,238,0},
{0,254,511,257,455,198,0},
{0,511,511,257,511,254,0},
{0,511,511,273,273,257,0},
{0,511,511,17,17,1,0},
{0,254,511,257,499,242,0},
{0,511,511,16,511,511,0},
{0,0,257,511,511,257,0},
{0,192,448,256,511,255,0},
{0,511,511,56,495,455,0},
{0,511,511,256,256,256,0},
{511,511,28,112,28,511,511},
{0,511,511,28,48,511,511},
{0,254,511,257,511,254,0},
{0,511,511,17,31,14,0},
{0,254,511,769,2047,1278,0},
{0,511,511,17,511,494,0},
{0,206,415,313,499,230,0},
{0,1,1,511,511,1,1},
{0,255,511,256,511,255,0},
{0,127,255,384,255,127,0},
{63,511,448,60,448,511,63},
{0,455,495,56,495,455,0},
{0,15,31,496,496,31,15},
{0,449,497,317,271,259,0},
{0,0,2047,2047,1025,1025,0},
{0,3,15,60,240,448,256},
{0,1025,1025,2047,2047,0,0},
{0,4,6,3,6,4,0},
{1024,1024,1024,1024,1024,1024,1024},
{0,0,3,7,4,0,0},
{0,192,488,296,504,496,0},
{0,511,511,264,504,240,0},
{0,240,504,264,408,144,0},
{0,240,504,264,511,511,0},
{0,240,504,296,312,176,0},
{0,8,510,511,9,9,0},
{0,624,1272,1160,2040,1016,0},
{0,511,511,8,504,496,0},
{0,0,264,505,505,256,0},
{0,1024,1032,2041,1017,0,0},
{0,511,511,112,472,392,0},
{0,0,257,511,511,256,0},
{504,504,24,240,24,504,496},
{0,504,504,8,504,496,0},
{0,240,504,264,504,240,0},
{0,2040,2040,264,504,240,0},
{0,240,504,264,2040,2040,0},
{0,504,504,16,24,24,0},
{0,304,376,360,488,200,0},
{0,8,254,510,264,256,0},
{0,248,504,256,504,504,0},
{0,120,248,384,248,120,0},
{120,504,384,240,384,504,120},
{0,408,504,96,504,408,0},
{0,1144,1784,896,504,120,0},
{0,392,456,360,312,280,0},
{0,16,254,495,257,257,0},
{0,0,2015,2015,0,0,0},
{0,257,495,254,16,0,0},
{0,6,3,7,6,3,0},
{0,496,408,396,408,496,0}
};

unsigned int code font_10x16[][10] = {
{0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,7166,0,0,0,0,0},
{0,0,30,0,0,0,30,0,0,0},
{256,6432,1952,376,6438,1952,376,294,288,0},
{0,0,6172,4134,4162,16383,4482,3842,0,0},
{4124,2082,1058,802,156,3648,4400,4360,4356,3586},
{1792,2176,4188,4194,4514,4914,7196,6144,5632,384},
{0,0,0,0,30,0,0,0,0,0},
{0,0,0,2016,6168,12300,8196,16386,16386,0},
{0,16386,16386,8196,12300,6168,2016,0,0,0},
{0,8,24,240,78,240,24,8,0,0},
{512,512,512,512,8160,512,512,512,512,0},
{0,0,0,38912,30720,0,0,0,0,0},
{0,0,256,256,256,256,256,256,0,0},
{0,0,0,6144,6144,0,0,0,0,0},
{0,16384,12288,3072,768,192,48,12,2,0},
{0,1008,3084,4098,4098,4098,3084,1008,0,0},
{0,4100,4100,4100,8190,4096,4096,4096,0,0},
{0,6150,5122,4610,4354,4290,4156,0,0,0},
{0,0,4098,4162,4162,4162,4028,0,0,0},
{768,704,544,536,516,8190,512,512,0,0},
{0,0,4158,4130,4130,2114,1922,0,0,0},
{0,2032,2124,4130,4130,4130,2114,1920,0,0},
{0,2,6146,1794,194,50,10,6,0,0},
{0,3868,4258,4162,4162,4258,2466,1564,0,0},
{0,120,4228,4354,4354,4354,3204,1016,0,0},
{0,0,0,6240,6240,0,0,0,0,0},
{0,0,0,39008,30816,0,0,0,0,0},
{0,0,768,768,1152,1152,2112,2112,4128,0},
{0,1152,1152,1152,1152,1152,1152,1152,1152,0},
{0,4128,2112,2112,1152,1152,768,768,0,0},
{0,14,2,6914,130,66,38,28,0,0},
{1008,3096,6148,5090,5138,5642,7050,2044,1024,1024},
{4096,3584,896,624,536,560,704,768,3072,4096},
{0,8184,4232,4232,4232,4232,4424,3632,0,0},
{960,3120,2064,4104,4104,4104,4104,4120,0,0},
{0,8184,4104,4104,4104,4104,2064,2016,0,0},
{0,8184,4360,4360,4360,4360,4360,4104,0,0},
{0,8184,264,264,264,264,264,8,0,0},
{0,960,3120,2064,4104,4104,4360,4360,7960,0},
{0,8184,256,256,256,256,256,8184,0,0},
{0,4104,4104,4104,8184,4104,4104,4104,0,0},
{0,0,4096,4104,4104,4104,4088,0,0,0},
{0,8184,128,384,576,1056,1040,2056,4096,0},
{0,8184,4096,4096,4096,4096,4096,4096,0,0},
{8184,56,480,1792,1536,448,56,8184,0,0},
{0,8184,16,96,384,1536,2048,8184,0,0},
{0,2016,2064,4104,4104,4104,4104,2064,2016,0},
{0,8184,264,264,264,264,136,112,0,0},
{0,2016,2064,4104,4104,4104,12296,18448,18400,0},
{0,8184,264,264,776,1160,2160,4096,0,0},
{0,6256,4168,4232,4232,4360,2312,3608,0,0},
{8,8,8,8,8184,8,8,8,8,0},
{0,2040,6144,4096,4096,4096,2048,2040,0,0},
{8,48,448,1536,6144,6144,1792,192,48,8},
{24,992,7168,3840,224,448,3584,7168,992,24},
{4104,2064,1056,576,384,384,576,1056,2064,4104},
{8,16,96,128,7936,128,64,32,16,8},
{0,6152,5128,4616,4360,4232,4168,4136,4120,0},
{0,0,0,32766,16386,16386,16386,16386,0,0},
{0,2,12,48,192,768,3072,12288,16384,0},
{0,16386,16386,16386,16386,32766,0,0,0,0},
{0,1024,768,224,56,14,112,896,1024,0},
{8192,8192,8192,8192,8192,8192,8192,8192,8192,8192},
{0,0,0,0,1,2,0,0,0,0},
{0,3072,4640,4384,4384,2336,8128,4096,0,0},
{0,8190,2112,4128,4128,4128,2144,1920,0,0},
{0,1920,2112,4128,4128,4128,4128,4128,0,0},
{0,1920,6208,4128,4128,4128,2112,8190,0,0},
{0,1920,2368,4384,4384,4384,4384,4544,0,0},
{0,32,32,8188,36,34,34,34,34,0},
{0,1920,38976,36896,36896,36896,18496,16352,0,0},
{0,8190,128,64,32,32,32,8128,0,0},
{0,32,32,38,8166,0,0,0,0,0},
{0,32768,32800,32800,32806,32742,0,0,0,0},
{0,8190,256,768,1152,1088,2112,4128,0,0},
{0,0,2,2,2,8190,0,0,0,0},
{8160,64,32,32,8128,64,32,32,8128,0},
{0,8160,192,64,32,32,32,8128,0,0},
{0,1920,2112,4128,4128,4128,2112,1920,0,0},
{0,65504,2112,4128,4128,4128,2144,1920,0,0},
{0,1920,6208,4128,4128,4128,2112,65504,0,0},
{0,0,8160,128,64,32,32,224,0,0},
{0,6336,4384,4384,4640,4640,3104,0,0,0},
{0,32,32,4088,4128,4128,4128,4128,0,0},
{0,4064,4096,4096,4096,2048,8160,0,0,0},
{32,448,1536,6144,4096,3072,768,192,32,0},
{96,1920,6144,3584,448,384,3584,6144,1920,96},
{0,4128,2112,1152,768,768,1152,2112,4128,0},
{32800,32960,33536,17920,14336,6144,1536,256,192,32},
{0,4128,6176,5152,4640,4384,4256,4192,4128,0},
{0,0,256,256,16124,16386,16386,16386,0,0},
{0,0,0,0,32766,0,0,0,0,0},
{0,0,16386,16386,16386,16124,256,256,0,0},
{768,128,128,128,256,256,512,512,512,384},
{0,8064,4288,4128,4112,4128,4288,8064,0,0}
};

unsigned int code font_12x16[][12] = {
{0,0,0,0,0,0,0,0,0,0,0,0},
{0,0,0,0,124,13311,13311,124,0,0,0,0},
{0,0,0,60,60,0,0,60,60,0,0,0},
{0,512,7696,8080,1008,638,7710,8080,1008,638,30,16},
{0,0,1144,3324,3276,16383,16383,3276,4044,1928,0,0},
{0,12288,14392,7224,3640,1792,896,448,14560,14448,14392,28},
{0,0,7936,16312,12796,8646,14306,7742,7196,13824,8704,0},
{0,0,0,0,39,63,31,0,0,0,0,0},
{0,0,0,1008,4092,8190,14343,8193,8193,0,0,0},
{0,0,0,8193,8193,14343,8190,4092,1008,0,0,0},
{0,0,3224,3768,992,4088,4088,992,3768,3224,0,0},
{0,0,384,384,384,4080,4080,384,384,384,0,0},
{0,0,0,0,47104,63488,30720,0,0,0,0,0},
{0,0,384,384,384,384,384,384,384,384,0,0},
{0,0,0,0,14336,14336,14336,0,0,0,0,0},
{0,6144,7168,3584,1792,896,448,224,112,56,28,14},
{0,2040,8190,7686,13059,12675,12483,12387,12339,6174,8190,2040},
{0,0,0,12300,12300,12302,16383,16383,12288,12288,12288,0},
{0,12316,14366,15367,15875,14083,13187,12739,12515,12407,12350,12316},
{0,3084,7182,14343,12483,12483,12483,12483,12483,14823,8062,3644},
{0,960,992,880,824,796,782,775,16383,16383,768,768},
{0,3135,7295,14435,12387,12387,12387,12387,12387,14563,8131,3971},
{0,4032,8176,14840,12508,12494,12487,12483,12483,14787,8064,3840},
{0,3,3,3,12291,15363,3843,963,243,63,15,3},
{0,3840,8124,14846,12519,12483,12483,12483,12519,14846,8124,3840},
{0,60,126,12519,12483,12483,14531,7363,3779,2023,1022,252},
{0,0,0,0,7280,7280,7280,0,0,0,0,0},
{0,0,0,0,40048,64624,31856,0,0,0,0,0},
{0,0,192,480,1008,1848,3612,7182,14343,12291,0,0},
{0,0,1632,1632,1632,1632,1632,1632,1632,1632,1632,0},
{0,0,12291,14343,7182,3612,1848,1008,480,192,0,0},
{0,28,30,7,3,14211,14275,227,119,62,28,0},
{0,4088,8190,6151,13299,14331,13851,14331,14331,13831,1022,504},
{0,14336,16128,2016,1788,1567,1567,1788,2016,16128,14336,0},
{0,16383,16383,12483,12483,12483,12483,12519,14846,8124,3840,0},
{0,1008,4092,7182,14343,12291,12291,12291,14343,7182,3084,0},
{0,16383,16383,12291,12291,12291,12291,14343,7182,4092,1008,0},
{0,16383,16383,12483,12483,12483,12483,12483,12483,12291,12291,0},
{0,16383,16383,195,195,195,195,195,195,3,3,0},
{0,1008,4092,7182,14343,12291,12483,12483,12483,16327,16326,0},
{0,16383,16383,192,192,192,192,192,192,16383,16383,0},
{0,0,0,12291,12291,16383,16383,12291,12291,0,0,0},
{0,3584,7680,14336,12288,12288,12288,12288,14336,8191,2047,0},
{0,16383,16383,192,480,1008,1848,3612,7182,14343,12291,0},
{0,16383,16383,12288,12288,12288,12288,12288,12288,12288,12288,0},
{0,16383,16383,30,120,480,480,120,30,16383,16383,0},
{0,16383,16383,14,56,240,960,1792,7168,16383,16383,0},
{0,1008,4092,7182,14343,12291,12291,14343,7182,4092,1008,0},
{0,16383,16383,387,387,387,387,387,455,254,124,0},
{0,1008,4092,7182,14343,12291,13827,15879,7182,16380,13296,0},
{0,16383,16383,387,387,899,1923,3971,7623,14590,12412,0},
{0,3132,7294,14567,12483,12483,12483,12483,14791,8078,3852,0},
{0,0,3,3,3,16383,16383,3,3,3,0,0},
{0,2047,8191,14336,12288,12288,12288,12288,14336,8191,2047,0},
{0,7,63,504,4032,15872,15872,4032,504,63,7,0},
{0,16383,16383,7168,1536,896,896,1536,7168,16383,16383,0},
{0,12291,15375,3612,816,480,480,816,3612,15375,12291,0},
{0,3,15,60,240,16320,16320,240,60,15,3,0},
{0,12291,15363,15875,13059,12739,12515,12339,12319,12303,12291,0},
{0,0,0,16383,16383,12291,12291,12291,12291,0,0,0},
{0,14,28,56,112,224,448,896,1792,3584,7168,6144},
{0,0,0,12291,12291,12291,12291,16383,16383,0,0,0},
{0,96,112,56,28,14,7,14,28,56,112,96},
{0,49152,49152,49152,49152,49152,49152,49152,49152,49152,49152,49152},
{0,0,0,0,0,62,126,78,0,0,0,0},
{0,7168,15936,13152,13152,13152,13152,13152,13152,16352,16320,0},
{0,16383,16383,12480,12384,12384,12384,12384,14560,8128,3968,0},
{0,3968,8128,14560,12384,12384,12384,12384,12384,6336,2176,0},
{0,3968,8128,14560,12384,12384,12384,12512,12480,16383,16383,0},
{0,3968,8128,15328,13152,13152,13152,13152,13152,5056,384,0},
{0,192,192,16380,16382,199,195,195,3,0,0,0},
{0,896,51136,52960,52320,52320,52320,52320,58976,32736,16352,0},
{0,16383,16383,192,96,96,96,224,16320,16256,0,0},
{0,0,0,12288,12384,16364,16364,12288,12288,0,0,0},
{0,0,0,24576,57344,49152,49248,65516,32748,0,0,0},
{0,0,16383,16383,768,1920,4032,7392,14432,12288,0,0},
{0,0,0,12288,12291,16383,16383,12288,12288,0,0,0},
{0,16352,16320,224,224,16320,16320,224,224,16320,16256,0},
{0,0,16352,16352,96,96,96,96,224,16320,16256,0},
{0,3968,8128,14560,12384,12384,12384,12384,14560,8128,3968,0},
{0,65504,65504,3168,6240,6240,6240,6240,7392,4032,1920,0},
{0,1920,4032,7392,6240,6240,6240,6240,3168,65504,65504,0},
{0,0,16352,16352,192,96,96,96,96,224,192,0},
{0,4544,13280,13152,13152,13152,13152,16224,7744,0,0,0},
{0,96,96,8190,16382,12384,12384,12384,12288,0,0,0},
{0,4064,8160,14336,12288,12288,12288,12288,6144,16352,16352,0},
{0,96,480,1920,7680,14336,14336,7680,1920,480,96,0},
{0,2016,8160,14336,7168,4064,4064,7168,14336,8160,2016,0},
{0,12384,14560,7616,3968,1792,3968,7616,14560,12384,0,0},
{0,0,96,33248,59264,32256,7680,1920,480,96,0,0},
{0,12384,14432,15456,13920,13152,12768,12512,12384,12320,0,0},
{0,0,128,448,8188,16254,28679,24579,24579,24579,0,0},
{0,0,0,0,0,16319,16319,0,0,0,0,0},
{0,0,24579,24579,24579,28679,16254,8188,448,128,0,0},
{0,16,24,12,4,12,24,16,24,12,4,0},
{0,3840,3968,3264,3168,3120,3120,3168,3264,3968,3840,0}
};*/