//*****************************************************************************
//  COPYRIGHT (C) 2010
//*****************************************************************************
//
//  PROJECT NAME : powermeter
//  FILENAME     : powermeter.c
//
//  COMPILER     : KEIL A51 Macro Assembler V6.20c
//
//  TARGET HW    : C8051F007
//
//  CREATION     : 22 May 2011
//
//  VERSION		: 1.0
//-----------------------------------------------------------------------------
//  DESCRIPTON   : 	
//******************************************************************************

#include "C8051F000.h"
#include "powermeter.h"
#include "lcd_3310_lib.h"
#include <stdio.h>
#include <math.h>


#define SAMPLES 128
#define OFFSET  2047

sfr16 adc0 = 0xbe; 


long int s_voltage, s_current;
bit data_ready;


// main program
void main (void){
	unsigned char buffer[10];
	int i, sc;

	unsigned long int voltage, current, power, complex_power;
	double cos_phi;


	// init stuff
	Init_Device();
	lcd_init();
	lcd_clear_screen();
	lcd_flush();

	// print title
	lcd_set_cursor(0,0);
	lcd_send_string("Powermeter", 0);
	lcd_draw_line(0, 9, 83, 9);

	data_ready = 0;


	// main loop
	while(1)
	{
		voltage = 0; current = 0; power = 0;
		TMR3CN |= 0x04; // start timer to gather samples
		for(sc = 0; sc < SAMPLES; ++sc) // integrate over 2 periods
		{
			while(!data_ready); // wait for new data
			
			s_voltage = (s_voltage - OFFSET) * 2; // in 10^-1 V
			s_current = (s_current - OFFSET) * 2; // in mA
			power += labs(s_voltage) * labs(s_current);  // in 10^-4 W
			current += (s_current * s_current);
			voltage += (s_voltage * s_voltage);
			data_ready = 0;
		}
		
		TMR3CN &= ~0x04; // stop timer 

		voltage = sqrt(voltage >> 7); // real power
		current = sqrt(current >> 7); // real power
		complex_power = voltage * current;  // complexe power
		power = power >> 7;

		if (power < 10000) // stops jumping cos(phi) @  P < 0.5W
			cos_phi = 1;
		else
			cos_phi = (double)power / complex_power;


		sprintf(buffer, "P: %5.1f W", (double)power/10000);
		lcd_set_cursor(0,12);
		lcd_send_string(buffer, 0);
		sprintf(buffer, "S: %5.1f VA",(double)complex_power/10000);
		lcd_set_cursor(0,21);
		lcd_send_string(buffer, 0);
		sprintf(buffer, "cos(^): %1.2f",cos_phi);
		lcd_set_cursor(0,30);
		lcd_send_string(buffer, 0);

		sprintf(buffer, "%5.1fV  %4.2fA ", (double)voltage/10, (double)current/1000);
		lcd_set_cursor(0,39);
		lcd_send_string(buffer, 0);

		lcd_flush();

	}
}	


void measure_routine() interrupt 14 //1.6kHz -> 32 samples / periode
{
	if (data_ready)
		while(1);
	TMR3CN &= ~0x80;
	AMX0SL = 0x00;
	ADBUSY = 1;
	while(!ADCINT);
	ADCINT = 0;
	s_voltage = (ADC0H << 8) | ADC0L;
	AMX0SL = 0x01;
	ADBUSY = 1;
	while(!ADCINT);
	ADCINT = 0;
	s_current = (ADC0H << 8) | ADC0L;
	data_ready = 1;
}


unsigned int get_adc_val()
{
	ADBUSY = 1;
	while(!ADCINT); // wait for sample
	ADCINT = 0;

	return (ADC0H << 8) | ADC0L;
}
